@@ -37,7 +37,7 @@ pub use lease::*;
 mod remote_forwarder;
 pub use remote_forwarder::*;
 
-pub use ockam_core::async_trait::async_trait as async_worker;
+pub use ockam_core::worker;
 pub use ockam_core::{
     Address, Any, Encoded, Error, Message, Result, Route, Routed, RouterMessage, TransportMessage,
     Worker,
